`default_nettype none
`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 02/15/2022 03:25:04 PM
// Design Name: 
// Module Name: top
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module top_dl(
   input  wire        clk,
   input  wire        rst,

   output wire        phy_rstn,
   output wire        mdc,
   inout  wire        mdio,
   input  wire        rxclk,
   input  wire [3:0]  rxd_i,
   input  wire        rxctrl_i,
   
   output wire [7:0]  led
);

assign phy_rstn = ~rst;
assign mdio = 1'bz;

reg [8:0] init_cntr;
always @(posedge clk)
if (rst)
   init_cntr <= 0;
else if (init_cntr[8]==1'b0)
   init_cntr <= init_cntr + 1;

// 7:3: mdc counter
//   2: mdc, 12.5 MHz
assign mdc = init_cntr[2];

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// TODO

// 200 MHz clock generation for IDELAYCNTRL
wire clk_200;
clk_gen clk_gen_i(
   .clk_i     (clk),
   .rst_i     (rst),
   .clk_200   (clk_200)
);


// IDELAYCNTRL
IDELAYCTRL IDELAYCTRL_i (
   .RDY(),       // 1-bit output: Ready output
   .REFCLK(clk_200), // 1-bit input: Reference clock input
   .RST(rst)        // 1-bit input: Active high reset input
);


// BUFIO
wire clk_bufio;
BUFIO BUFIO_inst (
   .O(clk_bufio), // 1-bit output: Clock output (connect to I/O clock loads).
   .I(rxclk)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);

// BUFR
wire clk_bufr;
BUFR #(
   .BUFR_DIVIDE("BYPASS"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
   .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_inst (
   .O(clk_bufr),     // 1-bit output: Clock output port
   .CE(1'b1),   // 1-bit input: Active high, clock enable (Divided modes only)
   .CLR(1'b0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
   .I(rxclk)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local interconnect
);

wire [3:0] rxd_dl;
(* mark_debug = "TRUE" *) wire [7:0] rx_data;

genvar IB;
// generate loop: instantiate 4 IDDRs for data
generate
for (IB=0; IB < 4; IB=IB+1)
begin: IDATA
  
   IDELAYE2 #(
      .CINVCTRL_SEL("FALSE"),          // Enable dynamic clock inversion (FALSE, TRUE)
      .DELAY_SRC("IDATAIN"),           // Delay input (IDATAIN, DATAIN)
      .HIGH_PERFORMANCE_MODE("FALSE"), // Reduced jitter ("TRUE"), Reduced power ("FALSE")
      .IDELAY_TYPE("FIXED"),           // FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
      .IDELAY_VALUE(13),                // Input delay tap setting (0-31)   8
      .PIPE_SEL("FALSE"),              // Select pipelined mode, FALSE, TRUE
      .REFCLK_FREQUENCY(200.0),        // IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
      .SIGNAL_PATTERN("DATA")          // DATA, CLOCK input signal
   )
   IDELAYE2_data (
      .CNTVALUEOUT(), // 5-bit output: Counter value output
      .DATAOUT(rxd_dl[IB]),         // 1-bit output: Delayed data output
      .C(1'b0),                     // 1-bit input: Clock input
      .CE(1'b0),                   // 1-bit input: Active high enable increment/decrement input
      .CINVCTRL(1'b0),       // 1-bit input: Dynamic clock inversion input
      .CNTVALUEIN(5'b0),   // 5-bit input: Counter value input
      .DATAIN(1'b0),           // 1-bit input: Internal delay data input
      .IDATAIN(rxd_i[IB]),         // 1-bit input: Data input from the I/O
      .INC(1'b0),                 // 1-bit input: Increment / Decrement tap delay input
      .LD(1'b0),                   // 1-bit input: Load IDELAY_VALUE input
      .LDPIPEEN(1'b0),       // 1-bit input: Enable PIPELINE register to load data input
      .REGRST(rst)            // 1-bit input: Active-high reset tap-delay input
   );

   IDDR #(
      .DDR_CLK_EDGE("SAME_EDGE_PIPELINED"), // "OPPOSITE_EDGE", "SAME_EDGE" 
                                //    or "SAME_EDGE_PIPELINED" 
      .INIT_Q1(1'b0), // Initial value of Q1: 1'b0 or 1'b1
      .INIT_Q2(1'b0), // Initial value of Q2: 1'b0 or 1'b1
      .SRTYPE("SYNC") // Set/Reset type: "SYNC" or "ASYNC" 
   ) IDDR_d (
      .Q1(rx_data[0+IB]), // 1-bit output for positive edge of clock
      .Q2(rx_data[4+IB]), // 1-bit output for negative edge of clock
      .C(clk_bufio),   // 1-bit clock input
      .CE(1'b1), // 1-bit clock enable input
      .D(rxd_dl[IB]),   // 1-bit DDR data input
      .R(1'b0),   // 1-bit reset
      .S(1'b0)    // 1-bit set
   );
end
endgenerate

wire rxctrl_dl;
IDELAYE2 #(
   .CINVCTRL_SEL("FALSE"),          // Enable dynamic clock inversion (FALSE, TRUE)
   .DELAY_SRC("IDATAIN"),           // Delay input (IDATAIN, DATAIN)
   .HIGH_PERFORMANCE_MODE("FALSE"), // Reduced jitter ("TRUE"), Reduced power ("FALSE")
   .IDELAY_TYPE("FIXED"),           // FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
   .IDELAY_VALUE(13),                // Input delay tap setting (0-31)   8
   .PIPE_SEL("FALSE"),              // Select pipelined mode, FALSE, TRUE
   .REFCLK_FREQUENCY(200.0),        // IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
   .SIGNAL_PATTERN("DATA")          // DATA, CLOCK input signal
)
IDELAYE2_ctrl (
   .CNTVALUEOUT(), // 5-bit output: Counter value output
   .DATAOUT(rxctrl_dl),         // 1-bit output: Delayed data output
   .C(1'b0),                     // 1-bit input: Clock input
   .CE(1'b0),                   // 1-bit input: Active high enable increment/decrement input
   .CINVCTRL(1'b0),       // 1-bit input: Dynamic clock inversion input
   .CNTVALUEIN(5'b0),   // 5-bit input: Counter value input
   .DATAIN(1'b0),           // 1-bit input: Internal delay data input
   .IDATAIN(rxctrl_i),         // 1-bit input: Data input from the I/O
   .INC(1'b0),                 // 1-bit input: Increment / Decrement tap delay input
   .LD(1'b0),                   // 1-bit input: Load IDELAY_VALUE input
   .LDPIPEEN(1'b0),       // 1-bit input: Enable PIPELINE register to load data input
   .REGRST(rst)            // 1-bit input: Active-high reset tap-delay input
);
(* mark_debug = "TRUE" *) wire rxen, rxerr;
IDDR #(
   .DDR_CLK_EDGE("SAME_EDGE_PIPELINED"), // "OPPOSITE_EDGE", "SAME_EDGE" 
                             //    or "SAME_EDGE_PIPELINED" 
   .INIT_Q1(1'b0), // Initial value of Q1: 1'b0 or 1'b1
   .INIT_Q2(1'b0), // Initial value of Q2: 1'b0 or 1'b1
   .SRTYPE("SYNC") // Set/Reset type: "SYNC" or "ASYNC" 
) IDDR_d (
   .Q1(rxen), // 1-bit output for positive edge of clock
   .Q2(rxerr), // 1-bit output for negative edge of clock
   .C(clk_bufio),   // 1-bit clock input
   .CE(1'b1), // 1-bit clock enable input
   .D(rxctrl_dl),   // 1-bit DDR data input
   .R(1'b0),   // 1-bit reset
   .S(1'b0)    // 1-bit set
);

(* mark_debug = "TRUE" *) reg [7:0] data_r;
(* mark_debug = "TRUE" *) reg rxen_r, rxerr_r;

always @ (posedge clk_bufr)
begin
   data_r  <= rx_data;
   rxen_r  <= rxen;
   rxerr_r <= rxerr;
end

assign led[0] = &data_r & rxen_r & rxerr_r;

(* mark_debug = "TRUE" *) reg rx_dv;
always @ (posedge clk_bufr)
if (rxen==1'b0)
   rx_dv <= 1'b0;
else if (data_r==8'hd5)
   rx_dv <= 1'b1;
   
   
(* mark_debug = "TRUE" *) reg [15:0] rx_cntr;
always @ (posedge clk_bufr)
if (rx_dv==1'b0)
   rx_cntr <= 16'b0;
else
   rx_cntr <= rx_cntr + 1;


endmodule
